@@ -38,11 +38,18 @@ static const struct Config_Tag configs_Log[] =
 	{ "sTraceFileName", String_Tag, ConfigureParams.Log.sTraceFileName },
 	{ "nTextLogLevel", Int_Tag, &ConfigureParams.Log.nTextLogLevel },
 	{ "nAlertDlgLogLevel", Int_Tag, &ConfigureParams.Log.nAlertDlgLogLevel },
-	{ "nNumberBase", Int_Tag, &ConfigureParams.Log.nNumberBase },
 	{ "bConfirmQuit", Bool_Tag, &ConfigureParams.Log.bConfirmQuit },
 	{ NULL , Error_Tag, NULL }
 };
 
+/* Used to load/save debugger options */
+static const struct Config_Tag configs_Debugger[] =
+{
+	{ "nNumberBase", Int_Tag, &ConfigureParams.Debugger.nNumberBase },
+	{ "nDisasmLines", Int_Tag, &ConfigureParams.Debugger.nDisasmLines },
+	{ "nMemdumpLines", Int_Tag, &ConfigureParams.Debugger.nMemdumpLines },
+	{ NULL , Error_Tag, NULL }
+};
 
 /* Used to load/save screen options */
 static const struct Config_Tag configs_Screen[] =
@@ -53,7 +60,7 @@ static const struct Config_Tag configs_Screen[] =
 	{ "bAllowOverscan", Bool_Tag, &ConfigureParams.Screen.bAllowOverscan },
 	{ "nSpec512Threshold", Int_Tag, &ConfigureParams.Screen.nSpec512Threshold },
 	{ "nForceBpp", Int_Tag, &ConfigureParams.Screen.nForceBpp },
-	{ "bZoomLowRes", Bool_Tag, &ConfigureParams.Screen.bZoomLowRes },
+	{ "bAspectCorrect", Bool_Tag, &ConfigureParams.Screen.bAspectCorrect },
 	{ "bUseExtVdiResolutions", Bool_Tag, &ConfigureParams.Screen.bUseExtVdiResolutions },
 	{ "nVdiWidth", Int_Tag, &ConfigureParams.Screen.nVdiWidth },
 	{ "nVdiHeight", Int_Tag, &ConfigureParams.Screen.nVdiHeight },
@@ -61,14 +68,8 @@ static const struct Config_Tag configs_Screen[] =
 	{ "bShowStatusbar", Bool_Tag, &ConfigureParams.Screen.bShowStatusbar },
 	{ "bShowDriveLed", Bool_Tag, &ConfigureParams.Screen.bShowDriveLed },
 	{ "bCaptureChange", Bool_Tag, &ConfigureParams.Screen.bCaptureChange },
-	{ "nWindowBorderPixelsTop", Int_Tag, &ConfigureParams.Screen.nWindowBorderPixelsTop },
-	{ "nWindowBorderPixelsLeft", Int_Tag, &ConfigureParams.Screen.nWindowBorderPixelsLeft },
-	{ "nWindowBorderPixelsRight", Int_Tag, &ConfigureParams.Screen.nWindowBorderPixelsRight },
-	{ "nWindowBorderPixelsBottom", Int_Tag, &ConfigureParams.Screen.nWindowBorderPixelsBottom },
-	{ "nFullScreenBorderPixelsTop", Int_Tag, &ConfigureParams.Screen.nFullScreenBorderPixelsTop },
-	{ "nFullScreenBorderPixelsLeft", Int_Tag, &ConfigureParams.Screen.nFullScreenBorderPixelsLeft },
-	{ "nFullScreenBorderPixelsRight", Int_Tag, &ConfigureParams.Screen.nFullScreenBorderPixelsRight },
-	{ "nFullScreenBorderPixelsBottom", Int_Tag, &ConfigureParams.Screen.nFullScreenBorderPixelsBottom },
+	{ "nMaxWidth", Int_Tag, &ConfigureParams.Screen.nMaxWidth },
+	{ "nMaxHeight", Int_Tag, &ConfigureParams.Screen.nMaxHeight },
 	{ NULL , Error_Tag, NULL }
 };
 
@@ -176,7 +177,7 @@ static const struct Config_Tag configs_ShortCutWithMod[] =
 {
 	{ "keyOptions",    Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_OPTIONS] },
 	{ "keyFullScreen", Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_FULLSCREEN] },
-	{ "keyMouseMode",  Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_MOUSEMODE] },
+	{ "keyMouseMode",  Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_MOUSEGRAB] },
 	{ "keyColdReset",  Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_COLDRESET] },
 	{ "keyWarmReset",  Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_WARMRESET] },
 	{ "keyScreenShot", Int_Tag, &ConfigureParams.Shortcut.withModifier[SHORTCUT_SCREENSHOT] },
@@ -200,7 +201,7 @@ static const struct Config_Tag configs_ShortCutWithoutMod[] =
 {
 	{ "keyOptions",    Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_OPTIONS] },
 	{ "keyFullScreen", Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_FULLSCREEN] },
-	{ "keyMouseMode",  Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_MOUSEMODE] },
+	{ "keyMouseMode",  Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_MOUSEGRAB] },
 	{ "keyColdReset",  Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_COLDRESET] },
 	{ "keyWarmReset",  Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_WARMRESET] },
 	{ "keyScreenShot", Int_Tag, &ConfigureParams.Shortcut.withoutModifier[SHORTCUT_SCREENSHOT] },
@@ -226,6 +227,7 @@ static const struct Config_Tag configs_Sound[] =
 	{ "bEnableSound", Bool_Tag, &ConfigureParams.Sound.bEnableSound },
 	{ "nPlaybackFreq", Int_Tag, &ConfigureParams.Sound.nPlaybackFreq },
 	{ "szYMCaptureFileName", String_Tag, ConfigureParams.Sound.szYMCaptureFileName },
+	{ "nSdlAudioBufferSize", Int_Tag, &ConfigureParams.Sound.SdlAudioBufferSize },
 	{ NULL , Error_Tag, NULL }
 };
 
@@ -260,13 +262,13 @@ static const struct Config_Tag configs_HardDisk[] =
 	{ "bBootFromHardDisk", Bool_Tag, &ConfigureParams.HardDisk.bBootFromHardDisk },
 	{ "bUseHardDiskDirectory", Bool_Tag, &ConfigureParams.HardDisk.bUseHardDiskDirectories },
 	{ "szHardDiskDirectory", String_Tag, ConfigureParams.HardDisk.szHardDiskDirectories[DRIVE_C] },
-	/*{ "szHardDiskDirD", String_Tag, ConfigureParams.HardDisk.szHardDiskDirectories[DRIVE_D] },*/
-	/*{ "szHardDiskDirE", String_Tag, ConfigureParams.HardDisk.szHardDiskDirectories[DRIVE_E] },*/
-	/*{ "szHardDiskDirF", String_Tag, ConfigureParams.HardDisk.szHardDiskDirectories[DRIVE_F] },*/
+	{ "nWriteProtection", Int_Tag, &ConfigureParams.HardDisk.nWriteProtection },
 	{ "bUseHardDiskImage", Bool_Tag, &ConfigureParams.HardDisk.bUseHardDiskImage },
 	{ "szHardDiskImage", String_Tag, ConfigureParams.HardDisk.szHardDiskImage },
-	{ "bUseIdeHardDiskImage", Bool_Tag, &ConfigureParams.HardDisk.bUseIdeHardDiskImage },
-	{ "szIdeHardDiskImage", String_Tag, ConfigureParams.HardDisk.szIdeHardDiskImage },
+	{ "bUseIdeMasterHardDiskImage", Bool_Tag, &ConfigureParams.HardDisk.bUseIdeMasterHardDiskImage },
+	{ "bUseIdeSlaveHardDiskImage", Bool_Tag, &ConfigureParams.HardDisk.bUseIdeSlaveHardDiskImage },
+	{ "szIdeMasterHardDiskImage", String_Tag, ConfigureParams.HardDisk.szIdeMasterHardDiskImage },
+	{ "szIdeSlaveHardDiskImage", String_Tag, ConfigureParams.HardDisk.szIdeSlaveHardDiskImage },
 	{ NULL , Error_Tag, NULL }
 };
 
@@ -337,14 +339,18 @@ void Configuration_SetDefault(void)
 	/* Clear parameters */
 	memset(&ConfigureParams, 0, sizeof(CNF_PARAMS));
 
-	/* Set defaults for logging and debugging */
+	/* Set defaults for logging and tracing */
 	strcpy(ConfigureParams.Log.sLogFileName, "stderr");
 	strcpy(ConfigureParams.Log.sTraceFileName, "stderr");
 	ConfigureParams.Log.nTextLogLevel = LOG_TODO;
 	ConfigureParams.Log.nAlertDlgLogLevel = LOG_ERROR;
-	ConfigureParams.Log.nNumberBase = 10;
 	ConfigureParams.Log.bConfirmQuit = true;
 
+	/* Set defaults for debugger */
+	ConfigureParams.Debugger.nNumberBase = 10;
+	ConfigureParams.Debugger.nDisasmLines = 8;
+	ConfigureParams.Debugger.nMemdumpLines = 8;
+
 	/* Set defaults for floppy disk images */
 	ConfigureParams.DiskImage.bAutoInsertDiskB = true;
 	ConfigureParams.DiskImage.bSlowFloppy = false;
@@ -359,6 +365,7 @@ void Configuration_SetDefault(void)
 
 	/* Set defaults for hard disks */
 	ConfigureParams.HardDisk.bBootFromHardDisk = false;
+	ConfigureParams.HardDisk.nWriteProtection = WRITEPROT_OFF;
 	ConfigureParams.HardDisk.nHardDiskDir = DRIVE_C;
 	ConfigureParams.HardDisk.bUseHardDiskDirectories = false;
 	for (i = 0; i < MAX_HARDDRIVES; i++)
@@ -368,8 +375,10 @@ void Configuration_SetDefault(void)
 	}
 	ConfigureParams.HardDisk.bUseHardDiskImage = false;
 	strcpy(ConfigureParams.HardDisk.szHardDiskImage, psWorkingDir);
-	ConfigureParams.HardDisk.bUseIdeHardDiskImage = false;
-	strcpy(ConfigureParams.HardDisk.szIdeHardDiskImage, psWorkingDir);
+	ConfigureParams.HardDisk.bUseIdeMasterHardDiskImage = false;
+	strcpy(ConfigureParams.HardDisk.szIdeMasterHardDiskImage, psWorkingDir);
+	ConfigureParams.HardDisk.bUseIdeSlaveHardDiskImage = false;
+	strcpy(ConfigureParams.HardDisk.szIdeSlaveHardDiskImage, psWorkingDir);
 
 	/* Set defaults for Joysticks */
 	for (i = 0; i < JOYSTICK_COUNT; i++)
@@ -401,7 +410,7 @@ void Configuration_SetDefault(void)
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_DEBUG] = SDLK_PAUSE;
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_OPTIONS] = SDLK_o;
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_FULLSCREEN] = SDLK_f;
-	ConfigureParams.Shortcut.withModifier[SHORTCUT_MOUSEMODE] = SDLK_m;
+	ConfigureParams.Shortcut.withModifier[SHORTCUT_MOUSEGRAB] = SDLK_m;
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_COLDRESET] = SDLK_c;
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_WARMRESET] = SDLK_r;
 	ConfigureParams.Shortcut.withModifier[SHORTCUT_SCREENSHOT] = SDLK_g;
@@ -446,7 +455,7 @@ void Configuration_SetDefault(void)
 	ConfigureParams.Screen.bAllowOverscan = true;
 	ConfigureParams.Screen.nSpec512Threshold = 16;
 	ConfigureParams.Screen.nForceBpp = 0;
-	ConfigureParams.Screen.bZoomLowRes = false;
+	ConfigureParams.Screen.bAspectCorrect = true;
 	ConfigureParams.Screen.nMonitorType = MONITOR_TYPE_RGB;
 	ConfigureParams.Screen.bUseExtVdiResolutions = false;
 	ConfigureParams.Screen.nVdiWidth = 640;
@@ -455,20 +464,16 @@ void Configuration_SetDefault(void)
 	ConfigureParams.Screen.bShowStatusbar = true;
 	ConfigureParams.Screen.bShowDriveLed = true;
 	ConfigureParams.Screen.bCaptureChange = false;
-	ConfigureParams.Screen.nWindowBorderPixelsTop = OVERSCAN_TOP;
-	ConfigureParams.Screen.nWindowBorderPixelsLeft = 48;
-	ConfigureParams.Screen.nWindowBorderPixelsRight = 48;
-	ConfigureParams.Screen.nWindowBorderPixelsBottom = MAX_OVERSCAN_BOTTOM-2;
-	ConfigureParams.Screen.nFullScreenBorderPixelsTop = OVERSCAN_TOP;
-	ConfigureParams.Screen.nFullScreenBorderPixelsLeft = 32;
-	ConfigureParams.Screen.nFullScreenBorderPixelsRight = 32;
-	ConfigureParams.Screen.nFullScreenBorderPixelsBottom = MAX_OVERSCAN_BOTTOM-2;
+	/* target 800x600 screen with statusbar out of screen */
+	ConfigureParams.Screen.nMaxWidth = 2*(48+320+48);
+	ConfigureParams.Screen.nMaxHeight = 2*NUM_VISIBLE_LINES+24;
 
 	/* Set defaults for Sound */
 	ConfigureParams.Sound.bEnableSound = true;
 	ConfigureParams.Sound.nPlaybackFreq = 44100;
 	sprintf(ConfigureParams.Sound.szYMCaptureFileName, "%s%chatari.wav",
 	        psWorkingDir, PATHSEP);
+	ConfigureParams.Sound.SdlAudioBufferSize = 0;
 
 	/* Set defaults for Rom */
 	sprintf(ConfigureParams.Rom.szTosImageFileName, "%s%ctos.img",
@@ -529,6 +534,16 @@ void Configuration_Apply(bool bReset)
 		nFrameSkips = ConfigureParams.Screen.nFrameSkips;
 	}
 
+	/* Sound settings */
+	/* SDL sound buffer in ms */
+	SdlAudioBufferSize = ConfigureParams.Sound.SdlAudioBufferSize;
+	if ( SdlAudioBufferSize == 0 )			/* use default setting for SDL */
+		;
+	else if ( SdlAudioBufferSize < 10 )		/* min of 10 ms */
+		SdlAudioBufferSize = 10;
+	else if ( SdlAudioBufferSize > 100 )		/* max of 100 ms */
+		SdlAudioBufferSize = 100;
+
 	/* Set playback frequency */
 	Audio_SetOutputAudioFreq(ConfigureParams.Sound.nPlaybackFreq);
 
@@ -586,7 +601,7 @@ static int Configuration_LoadSection(const char *pFilename, const struct Config_
 
 	if (ret < 0)
 		fprintf(stderr, "Can not load configuration file %s (section %s).\n",
-		        sConfigFileName, pSection);
+		        pFilename, pSection);
 
 	return ret;
 }
@@ -595,7 +610,7 @@ static int Configuration_LoadSection(const char *pFilename, const struct Config_
 /*-----------------------------------------------------------------------*/
 /**
  * Load program setting from configuration file. If psFileName is NULL, use
- * the default (i.e. the users) configuration file.
+ * the configuration file given in configuration / last selected by user.
  */
 void Configuration_Load(const char *psFileName)
 {
@@ -609,6 +624,7 @@ void Configuration_Load(const char *psFileName)
 	}
 
 	Configuration_LoadSection(psFileName, configs_Log, "[Log]");
+	Configuration_LoadSection(psFileName, configs_Debugger, "[Debugger]");
 	Configuration_LoadSection(psFileName, configs_Screen, "[Screen]");
 	Configuration_LoadSection(psFileName, configs_Joystick0, "[Joystick0]");
 	Configuration_LoadSection(psFileName, configs_Joystick1, "[Joystick1]");
@@ -642,7 +658,7 @@ static int Configuration_SaveSection(const char *pFilename, const struct Config_
 	ret = update_config(pFilename, configs, pSection);
 
 	if (ret < 0)
-		fprintf(stderr, "Error while updating section %s\n", pSection);
+		fprintf(stderr, "Error while updating section %s in %s\n", pSection, pFilename);
 
 	return ret;
 }
@@ -659,6 +675,7 @@ void Configuration_Save(void)
 		Log_AlertDlg(LOG_ERROR, "Error saving config file.");
 		return;
 	}
+	Configuration_SaveSection(sConfigFileName, configs_Debugger, "[Debugger]");
 	Configuration_SaveSection(sConfigFileName, configs_Screen, "[Screen]");
 	Configuration_SaveSection(sConfigFileName, configs_Joystick0, "[Joystick0]");
 	Configuration_SaveSection(sConfigFileName, configs_Joystick1, "[Joystick1]");
