@@ -28,6 +28,9 @@
 #include "math.h"
 #include "dsp_core.h"
 #include "dsp_cpu.h"
+#include "ioMem.h"
+#include "dsp.h"
+
 
 /*--- Defines ---*/
 
@@ -48,8 +51,10 @@
 static void dsp_core_dsp2host(dsp_core_t *dsp_core);
 static void dsp_core_host2dsp(dsp_core_t *dsp_core);
 
+static void (*dsp_host_interrupt)(void);   /* Function to trigger host interrupt */
+
 /* Init DSP emulation */
-void dsp_core_init(dsp_core_t *dsp_core)
+void dsp_core_init(dsp_core_t *dsp_core,  void (*host_interrupt)(void))
 {
 	int i;
 
@@ -57,6 +62,8 @@ void dsp_core_init(dsp_core_t *dsp_core)
 	fprintf(stderr, "Dsp: core init\n");
 #endif
 
+	dsp_host_interrupt = host_interrupt;
+
 	memset(dsp_core, 0, sizeof(dsp_core_t));
 
 	/* Initialize Y:rom[0x0100-0x01ff] with a sin table */
@@ -145,10 +152,9 @@ void dsp_core_reset(dsp_core_t *dsp_core)
 	dsp_core_shutdown(dsp_core);
 
 	/* Memory */
-	memset((void*)dsp_core->periph, 0,sizeof(dsp_core->periph));
-	memset(dsp_core->stack, 0,sizeof(dsp_core->stack));
-	memset(dsp_core->registers, 0,sizeof(dsp_core->registers));
-	memset(dsp_core->interrupt_table, 0,sizeof(dsp_core->interrupt_table));
+	memset((void*)dsp_core->periph, 0, sizeof(dsp_core->periph));
+	memset(dsp_core->stack, 0, sizeof(dsp_core->stack));
+	memset(dsp_core->registers, 0, sizeof(dsp_core->registers));
 	dsp_core->dsp_host_rtx = 0;
 	dsp_core->dsp_host_htx = 0;
 
@@ -162,10 +168,18 @@ void dsp_core_reset(dsp_core_t *dsp_core)
 	}
 
 	/* Interruptions */
+	memset((void*)dsp_core->interrupt_isPending, 0, sizeof(dsp_core->interrupt_isPending));
 	dsp_core->interrupt_state = DSP_INTERRUPT_NONE;
 	dsp_core->interrupt_instr_fetch = -1;
 	dsp_core->interrupt_save_pc = -1;
 	dsp_core->interrupt_counter = 0;
+	dsp_core->interrupt_pipeline_count = 0;
+	for (i=0;i<5;i++) {
+		dsp_core->interrupt_ipl[i] = 3;
+	}
+	for (i=5;i<12;i++) {
+		dsp_core->interrupt_ipl[i] = -1;
+	}
 
 	/* host port init, dsp side */
 	dsp_core->periph[DSP_SPACE_X][DSP_HOST_HSR]=(1<<DSP_HOST_HSR_HTDE);
@@ -177,10 +191,11 @@ void dsp_core_reset(dsp_core_t *dsp_core)
 
 	/* SSI registers */
 	dsp_core->periph[DSP_SPACE_X][DSP_SSI_SR]=1<<DSP_SSI_SR_TDE;
-	dsp_core->ssi.waitFrame = 1;
+	dsp_core->ssi.waitFrameTX = 1;
+	dsp_core->ssi.waitFrameRX = 1;
 	dsp_core->ssi.TX = 0;
 	dsp_core->ssi.RX = 0;
-	dsp_core->ssi.slot_in_frame = 0;
+	dsp_core->ssi.dspPlay_handshakeMode_frame = 0;
 	
 	/* Other hardware registers */
 	dsp_core->periph[DSP_SPACE_X][DSP_IPR]=0;
@@ -195,22 +210,34 @@ void dsp_core_reset(dsp_core_t *dsp_core)
 	dsp56k_init_cpu(dsp_core);
 }
 
-/* Post a new interrupt to the interrupt table */
-void dsp_core_add_interrupt(dsp_core_t *dsp_core, Uint32 inter)
-{
-	if (dsp_core->interrupt_table[inter] == 0) { 
-		dsp_core->interrupt_table[inter] = 1;
-		dsp_core->interrupt_counter ++;
-	}
-}
-
 /* 
 	SSI INTERFACE processing
 */
 
-/* Process SSI peripheral code */
-void dsp_core_process_ssi_interface(dsp_core_t *dsp_core)
+/* Set PortC data register : send a frame order to the DMA in handshake mode */
+void dsp_core_setPortCDataRegister(dsp_core_t *dsp_core, Uint32 value)
 {
+	/* if DSP Record is in handshake mode with DMA Play */
+	if ((dsp_core->periph[DSP_SPACE_X][DSP_PCDDR] & 0x10) == 0x10) {
+		if ((value & 0x10) == 0x10) {
+			dsp_core->ssi.waitFrameRX = 0;
+			DSP_SsiTransmit_SC1();
+
+		}
+	}
+
+	/* if DSP Play is in handshake mode with DMA Record, high or low frame sync */
+	/* to allow / disable transfer of the data */
+	if ((dsp_core->periph[DSP_SPACE_X][DSP_PCDDR] & 0x20) == 0x20) {
+		if ((value & 0x20) == 0x20) {
+			dsp_core->ssi.dspPlay_handshakeMode_frame = 1;
+			dsp_core->ssi.waitFrameTX = 0;
+		}
+		else {
+			dsp_core->ssi.dspPlay_handshakeMode_frame = 0;
+			DSP_SsiTransmit_SC2(0);
+		}
+	}
 }
 
 /* SSI set TX register */
@@ -219,6 +246,12 @@ void dsp_core_ssi_writeTX(dsp_core_t *dsp_core, Uint32 value)
 	/* Clear SSI TDE bit */
 	dsp_core->periph[DSP_SPACE_X][DSP_SSI_SR] &= 0xff-(1<<DSP_SSI_SR_TDE);
 	dsp_core->ssi.TX = value;
+
+	/* if DSP Play is in handshake mode with DMA Record, send frame sync */
+	/* to allow transfer of the data */
+	if (dsp_core->ssi.dspPlay_handshakeMode_frame) {
+		DSP_SsiTransmit_SC2(1);
+	}
 }
 
 /* SSI set TDE register (dummy write) */
@@ -231,64 +264,29 @@ void dsp_core_ssi_writeTSR(dsp_core_t *dsp_core)
 /* SSI get RX register */
 Uint32 dsp_core_ssi_readRX(dsp_core_t *dsp_core)
 {
-	/* Clear SSI receive interrupt */
+	/* Clear SSI RDF bit */
 	dsp_core->periph[DSP_SPACE_X][DSP_SSI_SR] &= 0xff-(1<<DSP_SSI_SR_RDF);
-
 	return dsp_core->ssi.RX;
 }
 
-/* SSI receive serial clock */
-void dsp_core_ssi_receive_serial_clock(dsp_core_t *dsp_core)
+
+/**
+ * SSI receive serial clock.
+ *
+ */
+void dsp_core_ssi_Receive_SC0(dsp_core_t *dsp_core)
 {
-	Uint32 value; // i, temp=0;
-
-	value = dsp_core->ssi.TX;
-
-	/* Transfer data from SSI to crossbar*/
-
-	/* adjust value to trasnmit size word */
-	value >>= (24 - dsp_core->ssi.cra_word_length);
-	value &= dsp_core->ssi.cra_word_mask;
-
-	/* if bit SHFD in CRB is set, swap data to transmit */
-/*	if (dsp_core->ssi.crb_shifter) {
-		for (i=0; i<dsp_core->ssi.cra_word_length; i++) {
-			temp += value & 1;
-			temp <<= 1;
-			value >>= 1;
-		}
-		value = temp;
-	}
-*/
-	/* Transmit the data */
-	if (dsp_core->ssi.crb_te && dsp_core->ssi.waitFrame == 0) {
-		/* Send value to crossbar */
-		dsp_core->ssi.transmit_value = value;
-
-		/* generate interrupt ? */
-		if (dsp_core->periph[DSP_SPACE_X][DSP_SSI_CRB] & (1<<DSP_SSI_CRB_TIE)) {
-			if (dsp_core->periph[DSP_SPACE_X][DSP_SSI_SR] & (1<<DSP_SSI_SR_TDE)) {
-				dsp_core_add_interrupt(dsp_core, DSP_INTER_SSI_TRX_DATA);
-			} else {
-				dsp_core_add_interrupt(dsp_core, DSP_INTER_SSI_TRX_DATA);
-			}
-		}
-	}else{
-		dsp_core->ssi.transmit_value = 0;
-	}
-
-	/* set TDE */
-	dsp_core->periph[DSP_SPACE_X][DSP_SSI_SR] |= (1<<DSP_SSI_SR_TDE);
+	Uint32 value, i, temp=0;
 
 	/* Receive data from crossbar to SSI */
+	value = dsp_core->ssi.received_value;
 
 	/* adjust value to receive size word */
-	value = dsp_core->ssi.received_value;
 	value <<= (24 - dsp_core->ssi.cra_word_length);
 	value &= 0xffffff;
 
 	/* if bit SHFD in CRB is set, swap received data */
-/*	if (dsp_core->ssi.crb_shifter) {
+	if (dsp_core->ssi.crb_shifter) {
 		temp=0;
 		for (i=0; i<dsp_core->ssi.cra_word_length; i++) {
 			temp += value & 1;
@@ -297,41 +295,62 @@ void dsp_core_ssi_receive_serial_clock(dsp_core_t *dsp_core)
 		}
 		value = temp;
 	}
-*/
-	dsp_core->ssi.RX = value;
 
-	/* generate interrupt ? */
-	if (dsp_core->periph[DSP_SPACE_X][DSP_SSI_CRB] & (1<<DSP_SSI_CRB_RIE)) {
-		if (dsp_core->periph[DSP_SPACE_X][DSP_SSI_SR] & (1<<DSP_SSI_SR_RDF)) {
-			dsp_core_add_interrupt(dsp_core, DSP_INTER_SSI_RCV_DATA_E);
-		} else {
-			dsp_core_add_interrupt(dsp_core, DSP_INTER_SSI_RCV_DATA);
+	if (dsp_core->ssi.crb_re && dsp_core->ssi.waitFrameRX == 0) {
+		/* Send value to DSP receive */
+		dsp_core->ssi.RX = value;
+
+		/* generate interrupt ? */
+		if (dsp_core->periph[DSP_SPACE_X][DSP_SSI_CRB] & (1<<DSP_SSI_CRB_RIE)) {
+			if (dsp_core->periph[DSP_SPACE_X][DSP_SSI_SR] & (1<<DSP_SSI_SR_RDF)) {
+				dsp_add_interrupt(DSP_INTER_SSI_RCV_DATA);
+			} else {
+				dsp_add_interrupt(DSP_INTER_SSI_RCV_DATA);
+			}
 		}
+	}else{
+		dsp_core->ssi.RX = 0;
 	}
-	/* Set SSI receive */
+
+	/* set RDF */
 	dsp_core->periph[DSP_SPACE_X][DSP_SSI_SR] |= 1<<DSP_SSI_SR_RDF;
 }
 
-/* SSI generate internal clock */
-void dsp_core_ssi_generate_internal_clock(dsp_core_t *dsp_core)
+/**
+ * SSI receive SC1 bit : frame sync for receiver
+ *     value = 1 : beginning of a new frame
+ *     value = 0 : not beginning of a new frame
+ */
+void dsp_core_ssi_Receive_SC1(dsp_core_t *dsp_core, Uint32 value)
 {
-	/* TODO : write an internal timer */
-}
-
-/* SSI receive SC2 bit : frame sync */
-void dsp_core_ssi_receive_SC2(dsp_core_t *dsp_core, Uint32 value)
-{
-	dsp_core->ssi.slot_in_frame ++;
-	if (dsp_core->ssi.slot_in_frame >= dsp_core->ssi.cra_frame_rate_divider) {
-		dsp_core->ssi.slot_in_frame = 0;
-	}
-
 	/* SSI runs in network mode ? */
 	if (dsp_core->ssi.crb_mode) {
-		/* Detect Begin of a new frame */
-		if (dsp_core->ssi.slot_in_frame == 0) {
+		if (value) {
+			/* Beginning of a new frame */
+			dsp_core->periph[DSP_SPACE_X][DSP_SSI_SR] |= (1<<DSP_SSI_SR_RFS);
+			dsp_core->ssi.waitFrameRX = 0;
+		}else{
+			dsp_core->periph[DSP_SPACE_X][DSP_SSI_SR] &= 0xff-(1<<DSP_SSI_SR_RFS);
+		}
+	}else{
+		/* SSI runs in normal mode */
+		dsp_core->periph[DSP_SPACE_X][DSP_SSI_SR] |= (1<<DSP_SSI_SR_RFS);
+	}
+}
+
+/**
+ * SSI receive SC2 bit : frame sync for transmitter
+ *     value = 1 : beginning of a new frame
+ *     value = 0 : not beginning of a new frame
+ */
+void dsp_core_ssi_Receive_SC2(dsp_core_t *dsp_core, Uint32 value)
+{
+	/* SSI runs in network mode ? */
+	if (dsp_core->ssi.crb_mode) {
+		if (value) {
+			/* Beginning of a new frame */
 			dsp_core->periph[DSP_SPACE_X][DSP_SSI_SR] |= (1<<DSP_SSI_SR_TFS);
-			dsp_core->ssi.waitFrame = 0;
+			dsp_core->ssi.waitFrameTX = 0;
 		}else{
 			dsp_core->periph[DSP_SPACE_X][DSP_SSI_SR] &= 0xff-(1<<DSP_SSI_SR_TFS);
 		}
@@ -341,10 +360,58 @@ void dsp_core_ssi_receive_SC2(dsp_core_t *dsp_core, Uint32 value)
 	}
 }
 
-/* SSI SSI initialisations and state management */
+/**
+ * SSI transmit serial clock.
+ *
+ */
+void dsp_core_ssi_Receive_SCK(dsp_core_t *dsp_core)
+{
+	Uint32 value, i, temp=0;
+
+	value = dsp_core->ssi.TX;
+
+	/* Transfer data from SSI to crossbar*/
+
+	/* adjust value to transnmit size word */
+	value >>= (24 - dsp_core->ssi.cra_word_length);
+	value &= dsp_core->ssi.cra_word_mask;
+
+	/* if bit SHFD in CRB is set, swap data to transmit */
+	if (dsp_core->ssi.crb_shifter) {
+		for (i=0; i<dsp_core->ssi.cra_word_length; i++) {
+			temp += value & 1;
+			temp <<= 1;
+			value >>= 1;
+		}
+		value = temp;
+	}
+
+	/* Transmit the data */
+	if (dsp_core->ssi.crb_te && dsp_core->ssi.waitFrameTX == 0) {
+		/* Send value to crossbar */
+		dsp_core->ssi.transmit_value = value;
+
+		/* generate interrupt ? */
+		if (dsp_core->periph[DSP_SPACE_X][DSP_SSI_CRB] & (1<<DSP_SSI_CRB_TIE)) {
+			if (dsp_core->periph[DSP_SPACE_X][DSP_SSI_SR] & (1<<DSP_SSI_SR_TDE)) {
+				dsp_add_interrupt(DSP_INTER_SSI_TRX_DATA);
+			} else {
+				dsp_add_interrupt(DSP_INTER_SSI_TRX_DATA);
+			}
+		}
+	}else{
+		dsp_core->ssi.transmit_value = 0;
+	}
+
+	/* set TDE */
+	dsp_core->periph[DSP_SPACE_X][DSP_SSI_SR] |= (1<<DSP_SSI_SR_TDE);
+}
+
+
+/* SSI initialisations and state management */
 void dsp_core_ssi_configure(dsp_core_t *dsp_core, Uint32 adress, Uint32 value)
 {
-	Uint32 crb_te;
+	Uint32 crb_te, crb_re;
 
 	switch (adress) {
 		case DSP_SSI_CRA:
@@ -374,6 +441,7 @@ void dsp_core_ssi_configure(dsp_core_t *dsp_core, Uint32 adress, Uint32 value)
 			break;
 		case DSP_SSI_CRB:
 			crb_te = dsp_core->periph[DSP_SPACE_X][DSP_SSI_CRB] & (1<<DSP_SSI_CRB_TE);
+			crb_re = dsp_core->periph[DSP_SPACE_X][DSP_SSI_CRB] & (1<<DSP_SSI_CRB_RE);
 			dsp_core->periph[DSP_SPACE_X][DSP_SSI_CRB] = value;
 	
 			dsp_core->ssi.crb_src_clock = (value>>DSP_SSI_CRB_SCKD) & 1;
@@ -386,23 +454,20 @@ void dsp_core_ssi_configure(dsp_core_t *dsp_core, Uint32 adress, Uint32 value)
 			dsp_core->ssi.crb_rie       = (value>>DSP_SSI_CRB_RIE) & 1;
 
 			if (crb_te == 0 && dsp_core->ssi.crb_te) {
-				dsp_core->ssi.waitFrame = 1;
+				dsp_core->ssi.waitFrameTX = 1;
+			}
+			if (crb_re == 0 && dsp_core->ssi.crb_re) {
+				dsp_core->ssi.waitFrameRX = 1;
 			}
 			break;
 	}
 }
 
+
 /* 
 	HOST INTERFACE processing
 */
 
-/* Process Host Interface peripheral code */
-void dsp_core_process_host_interface(dsp_core_t *dsp_core)
-{
-	dsp_core_host2dsp(dsp_core);
-	dsp_core_dsp2host(dsp_core);
-}
-
 static void dsp_core_hostport_update_trdy(dsp_core_t *dsp_core)
 {
 	int trdy;
@@ -418,7 +483,14 @@ static void dsp_core_hostport_update_hreq(dsp_core_t *dsp_core)
 {
 	int hreq;
 
-	hreq = (dsp_core->hostport[CPU_HOST_ICR] & 0x3) & (dsp_core->hostport[CPU_HOST_ISR] & 0x3);
+	hreq = (dsp_core->hostport[CPU_HOST_ICR] & dsp_core->hostport[CPU_HOST_ISR]) & 0x3;
+
+	/* Trigger host interrupt? */
+	if (hreq && (dsp_core->hostport[CPU_HOST_ISR] & (1<<CPU_HOST_ISR_HREQ)) == 0) {
+		dsp_host_interrupt();
+	}
+
+	/* Set HREQ bit in hostport */
 	dsp_core->hostport[CPU_HOST_ISR] &= 0x7f;
 	dsp_core->hostport[CPU_HOST_ISR] |= (hreq?1:0) << CPU_HOST_ISR_HREQ;
 } 
@@ -445,7 +517,7 @@ static void dsp_core_dsp2host(dsp_core_t *dsp_core)
 
 	/* Is there an interrupt to send ? */
 	if (dsp_core->periph[DSP_SPACE_X][DSP_HOST_HCR] & (1<<DSP_HOST_HCR_HTIE)) {
-		dsp_core_add_interrupt(dsp_core, DSP_INTER_HOST_TRX_DATA);
+		dsp_add_interrupt(DSP_INTER_HOST_TRX_DATA);
 	}
 
 	/* Set RXDF bit to say that host can read */
@@ -479,7 +551,7 @@ static void dsp_core_host2dsp(dsp_core_t *dsp_core)
 
 	/* Is there an interrupt to send ? */
 	if (dsp_core->periph[DSP_SPACE_X][DSP_HOST_HCR] & (1<<DSP_HOST_HCR_HRIE)) {
-		dsp_core_add_interrupt(dsp_core, DSP_INTER_HOST_RCV_DATA);
+		dsp_add_interrupt(DSP_INTER_HOST_RCV_DATA);
 	}
 
 	/* Set TXDE bit to say that host can write */
@@ -501,6 +573,7 @@ void dsp_core_hostport_dspread(dsp_core_t *dsp_core)
 	fprintf(stderr, "Dsp: (H->D): Dsp HRDF cleared\n");
 #endif
 	dsp_core_hostport_update_trdy(dsp_core);
+	dsp_core_host2dsp(dsp_core);
 }
 
 void dsp_core_hostport_dspwrite(dsp_core_t *dsp_core)
@@ -510,6 +583,8 @@ void dsp_core_hostport_dspwrite(dsp_core_t *dsp_core)
 #if DSP_DISASM_HOSTWRITE
 	fprintf(stderr, "Dsp: (D->H): Dsp HTDE cleared\n");
 #endif
+
+	dsp_core_dsp2host(dsp_core);
 }
 
 /* Read/writes on host port */
@@ -521,6 +596,7 @@ Uint8 dsp_core_read_host(dsp_core_t *dsp_core, int addr)
 	if (addr == CPU_HOST_TRXL) {
 		/* Clear RXDF bit to say that CPU has read */
 		dsp_core->hostport[CPU_HOST_ISR] &= 0xff-(1<<CPU_HOST_ISR_RXDF);
+		dsp_core_dsp2host(dsp_core);
 		dsp_core_hostport_update_hreq(dsp_core);
 #if DSP_DISASM_HOSTREAD
 		fprintf(stderr, "Dsp: (D->H): Host RXDF=0\n");
@@ -548,7 +624,7 @@ void dsp_core_write_host(dsp_core_t *dsp_core, int addr, Uint8 value)
 				dsp_core->periph[DSP_SPACE_X][DSP_HOST_HSR] |= (1<<DSP_HOST_HSR_HCP);
 				/* Is there an interrupt to send ? */
 				if (dsp_core->periph[DSP_SPACE_X][DSP_HOST_HCR] & (1<<DSP_HOST_HCR_HCIE)) {
-					dsp_core_add_interrupt(dsp_core, DSP_INTER_HOST_COMMAND);
+					dsp_add_interrupt(DSP_INTER_HOST_COMMAND);
 				}
 			}
 			else{
@@ -606,7 +682,7 @@ void dsp_core_write_host(dsp_core_t *dsp_core, int addr, Uint8 value)
 
 					/* Is there an interrupt to send ? */
 					if (dsp_core->periph[DSP_SPACE_X][DSP_HOST_HCR] & (1<<DSP_HOST_HCR_HRIE)) {
-						dsp_core_add_interrupt(dsp_core, DSP_INTER_HOST_RCV_DATA);
+						dsp_add_interrupt(DSP_INTER_HOST_RCV_DATA);
 					}
 #if DSP_DISASM_HOSTWRITE
 					fprintf(stderr, "Dsp: (H->D): Dsp HRDF set\n");
@@ -621,6 +697,7 @@ void dsp_core_write_host(dsp_core_t *dsp_core, int addr, Uint8 value)
 #endif
 				}
 				dsp_core_hostport_update_trdy(dsp_core);
+				dsp_core_host2dsp(dsp_core);
 			}
 			break;
 	}
