@@ -18,6 +18,7 @@ const char Options_fileid[] = "Hatari options.c : " __DATE__ " " __TIME__;
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
+#include <SDL.h>
 
 #include "main.h"
 #include "options.h"
@@ -31,6 +32,7 @@ const char Options_fileid[] = "Hatari options.c : " __DATE__ " " __TIME__;
 #include "vdi.h"
 #include "joy.h"
 #include "log.h"
+#include "avi_record.h"
 
 #include "hatari-glue.h"
 
@@ -38,7 +40,9 @@ const char Options_fileid[] = "Hatari options.c : " __DATE__ " " __TIME__;
 bool bLoadAutoSave;        /* Load autosave memory snapshot at startup */
 bool bLoadMemorySave;      /* Load memory snapshot provided via option at startup */
 bool bBiosIntercept;       /* whether UAE should intercept Bios & XBios calls */
+bool AviRecordOnStartup;   /* Start avi recording at startup */
 
+static bool bNoSDLParachute;
 
 /*  List of supported options. */
 enum {
@@ -54,8 +58,11 @@ enum {
 	OPT_WINDOW,
 	OPT_GRAB,
 	OPT_ZOOM,
-	OPT_FRAMESKIPS,
+	OPT_MAXWIDTH,
+	OPT_MAXHEIGHT,
+	OPT_ASPECT,
 	OPT_BORDERS,
+	OPT_FRAMESKIPS,
 	OPT_STATUSBAR,
 	OPT_DRIVE_LED,
 	OPT_SPEC512,
@@ -64,6 +71,11 @@ enum {
 	OPT_VDI_PLANES,
 	OPT_VDI_WIDTH,
 	OPT_VDI_HEIGHT,
+	OPT_AVIRECORD,          /* record options */
+	OPT_AVIRECORD_VCODEC,
+	OPT_AVIRECORD_FPS,
+	OPT_AVIRECORD_CROP,
+	OPT_AVIRECORD_FILE,
 	OPT_JOYSTICK,		/* device options */
 	OPT_JOYSTICK0,
 	OPT_JOYSTICK1,
@@ -79,9 +91,12 @@ enum {
 	OPT_DISKA,		/* disk options */
 	OPT_DISKB,
 	OPT_SLOWFLOPPY,
+	OPT_WRITEPROT_FLOPPY,
+	OPT_WRITEPROT_HD,
 	OPT_HARDDRIVE,
 	OPT_ACSIHDIMAGE,
-	OPT_IDEHDIMAGE,
+	OPT_IDEMASTERHDIMAGE,
+	OPT_IDESLAVEHDIMAGE,
 	OPT_MEMSIZE,		/* memory options */
 	OPT_TOS,
 	OPT_CARTRIDGE,
@@ -94,11 +109,15 @@ enum {
 	OPT_TIMERD,
 	OPT_DSP,
 	OPT_SOUND,
+	OPT_SOUNDBUFFERSIZE,
 	OPT_KEYMAPFILE,
 	OPT_DEBUG,		/* debug options */
 	OPT_BIOSINTERCEPT,
 	OPT_TRACE,
 	OPT_TRACEFILE,
+	OPT_PARSE,
+	OPT_SAVECONFIG,
+	OPT_PARACHUTE,
 	OPT_CONTROLSOCKET,
 	OPT_LOGFILE,
 	OPT_LOGLEVEL,
@@ -143,11 +162,17 @@ static const opt_t HatariOptions[] = {
 	{ OPT_GRAB, NULL, "--grab",
 	  NULL, "Grab mouse (also) in window mode" },
 	{ OPT_ZOOM, "-z", "--zoom",
-	  "<x>", "Double ST low resolution (1=no, 2=yes)" },
+	  "<x>", "Double small resolutions (1=no, 2=yes)" },
+	{ OPT_MAXWIDTH, NULL, "--max-width",
+	  "<x>", "Maximum window width for zooming (Falcon/TT only)" },
+	{ OPT_MAXHEIGHT, NULL, "--max-height",
+	  "<x>", "Maximum window height for zooming (Falcon/TT only)" },
+	{ OPT_ASPECT, NULL, "--aspect",
+	  "<bool>", "Monitor aspect ratio correction (Falcon/TT only)" },
+	{ OPT_BORDERS, NULL, "--borders",
+	  "<bool>", "Show ST/STE screen borders (for overscan demos etc)" },
 	{ OPT_FRAMESKIPS, NULL, "--frameskips",
 	  "<x>", "Skip <x> frames after each shown frame (0=off, >4=auto/max)" },
-	{ OPT_BORDERS, NULL, "--borders",
-	  "<bool>", "Show screen borders (for overscan demos etc)" },
 	{ OPT_STATUSBAR, NULL, "--statusbar",
 	  "<bool>", "Show statusbar (floppy leds etc)" },
 	{ OPT_DRIVE_LED,   NULL, "--drive-led",
@@ -156,22 +181,36 @@ static const opt_t HatariOptions[] = {
 	  "<x>", "Spec512 palette threshold (0 <= x <= 512, 0=disable)" },
 	{ OPT_FORCEBPP, NULL, "--bpp",
 	  "<x>", "Force internal bitdepth (x = 8/15/16/32, 0=disable)" },
-	
+
 	{ OPT_HEADER, NULL, NULL, NULL, "VDI" },
 	{ OPT_VDI,	NULL, "--vdi",
 	  "<bool>", "Whether to use VDI screen mode" },
 	{ OPT_VDI_PLANES,NULL, "--vdi-planes",
 	  "<x>", "VDI mode bit-depth (x = 1/2/4)" },
 	{ OPT_VDI_WIDTH,     NULL, "--vdi-width",
-	  "<w>", "VDI mode width (320 < w <= 1024)" },
+	  "<w>", "VDI mode width (320 < w <= 1280)" },
 	{ OPT_VDI_HEIGHT,     NULL, "--vdi-height",
-	  "<h>", "VDI mode height (200 < h <= 768)" },
-	
+	  "<h>", "VDI mode height (200 < h <= 960)" },
+
+	{ OPT_HEADER, NULL, NULL, NULL, "Video recording" },
+	{ OPT_AVIRECORD, NULL, "--avirecord",
+	  NULL, "Start AVI recording" },
+	{ OPT_AVIRECORD_VCODEC, NULL, "--avi-vcodec",
+	  "<x>", "Select avi video codec (x = bmp/png)" },
+	{ OPT_AVIRECORD_FPS, NULL, "--avi-fps",
+	  "<x>", "Force avi frame rate (x = 50/60/71/...)" },
+	{ OPT_AVIRECORD_CROP, NULL, "--avi-crop",
+	  "<bool>", "Remove status bar from the recorded file" },
+	{ OPT_AVIRECORD_FILE, NULL, "--avi-file",
+	  "<file>", "Use <file> to record avi" },
+
 	{ OPT_HEADER, NULL, NULL, NULL, "Devices" },
 	{ OPT_JOYSTICK,  "-j", "--joystick",
 	  "<port>", "Emulate joystick with cursor keys in given port (0-5)" },
 	/* these have to be exactly the same as I'm relying compiler giving
 	 * them the same same string pointer when strings are identical
+	 * (Opt_ShowHelpSection() skips successive options with same help
+	 * pointer).
 	 */
 	{ OPT_JOYSTICK0, NULL, "--joy<port>",
 	  "<type>", "Set joystick type (none/keys/real) for given port" },
@@ -203,12 +242,18 @@ static const opt_t HatariOptions[] = {
 	  "<file>", "Set disk image for floppy drive B" },
 	{ OPT_SLOWFLOPPY,   NULL, "--slowfdc",
 	  "<bool>", "Slow down floppy disk access emulation" },
+	{ OPT_WRITEPROT_FLOPPY, NULL, "--protect-floppy",
+	  "<x>", "Write protect floppy image contents (on/off/auto)" },
+	{ OPT_WRITEPROT_HD, NULL, "--protect-hd",
+	  "<x>", "Write protect harddrive <dir> contents (on/off/auto)" },
 	{ OPT_HARDDRIVE, "-d", "--harddrive",
-	  "<dir>", "Emulate harddrive (partitions) with <dir> contents" },
+	  "<dir>", "Emulate harddrive partition(s) with <dir> contents" },
 	{ OPT_ACSIHDIMAGE,   NULL, "--acsi",
 	  "<file>", "Emulate an ACSI harddrive with an image <file>" },
-	{ OPT_IDEHDIMAGE,   NULL, "--ide",
-	  "<file>", "Emulate an IDE harddrive with an image <file>" },
+	{ OPT_IDEMASTERHDIMAGE,   NULL, "--ide-master",
+	  "<file>", "Emulate an IDE master harddrive with an image <file>" },
+	{ OPT_IDESLAVEHDIMAGE,   NULL, "--ide-slave",
+	  "<file>", "Emulate an IDE slave harddrive with an image <file>" },
 	
 	{ OPT_HEADER, NULL, NULL, NULL, "Memory" },
 	{ OPT_MEMSIZE,   "-s", "--memsize",
@@ -239,6 +284,8 @@ static const opt_t HatariOptions[] = {
 	  "<x>", "DSP emulation (x = none/dummy/emu, Falcon only)" },
 	{ OPT_SOUND,   NULL, "--sound",
 	  "<x>", "Sound frequency (x=off/6000-50066, off=fastest)" },
+	{ OPT_SOUNDBUFFERSIZE,   NULL, "--sound-buffer-size",
+	  "<x>", "Sound buffer size for SDL in ms (x=0/10-100, 0=use default buffer size)" },
 	{ OPT_KEYMAPFILE, "-k", "--keymap",
 	  "<file>", "Read (additional) keyboard mappings from <file>" },
 	
@@ -251,6 +298,12 @@ static const opt_t HatariOptions[] = {
 	  "<trace1,...>", "Activate emulation tracing, see '--trace help'" },
 	{ OPT_TRACEFILE, NULL, "--trace-file",
 	  "<file>", "Save trace output to <file> (default=stderr)" },
+	{ OPT_PARSE, NULL, "--parse",
+	  "<file>", "Parse/execute debugger commands from <file>" },
+	{ OPT_SAVECONFIG, NULL, "--saveconfig",
+	  NULL, "Save current Hatari configuration and exit" },
+	{ OPT_PARACHUTE, NULL, "--no-parachute",
+	  NULL, "Disable SDL parachute to get Hatari core dumps" },
 #if HAVE_UNIX_DOMAIN_SOCKETS
 	{ OPT_CONTROLSOCKET, NULL, "--control-socket",
 	  "<file>", "Hatari reads options from given socket at run-time" },
@@ -518,7 +571,7 @@ static int Opt_CheckBracketValue(const opt_t *opt, const char *str)
 		return OPT_CONTINUE;
 	}
 	offset = bracket - opt->str;
-	if (strlen(str) != offset + 1)
+	if (strncmp(opt->str, str, offset) != 0)
 	{
 		return OPT_CONTINUE;
 	}
@@ -558,37 +611,39 @@ static int Opt_WhichOption(int argc, const char *argv[], int idx)
 
 	for (opt = HatariOptions; opt->id != OPT_ERROR; opt++)
 	{	
-		if ((opt->str && !strcmp(str, opt->str)) ||
-		    (opt->chr && !strcmp(str, opt->chr)))
+		/* exact option name matches? */
+		if (!((opt->str && !strcmp(str, opt->str)) ||
+		      (opt->chr && !strcmp(str, opt->chr))))
 		{
-			
-			if (opt->arg)
+			/* no, maybe name<digit> matches? */
+			id = Opt_CheckBracketValue(opt, str);
+			if (id == OPT_CONTINUE)
 			{
-				if (idx+1 >= argc)
+				continue;
+			}
+			if (id == OPT_ERROR)
+			{
+				break;
+			}
+		}
+		/* matched, check args */
+		if (opt->arg)
+		{
+			if (idx+1 >= argc)
+			{
+				Opt_ShowError(opt->id, NULL, "Missing argument");
+				return OPT_ERROR;
+			}
+			/* early check for bools */
+			if (strcmp(opt->arg, "<bool>") == 0)
+			{
+				if (!Opt_Bool(argv[idx+1], opt->id, NULL))
 				{
-					Opt_ShowError(opt->id, NULL, "Missing argument");
 					return OPT_ERROR;
 				}
-				/* early check for bools */
-				if (strcmp(opt->arg, "<bool>") == 0)
-				{
-					if (!Opt_Bool(argv[idx+1], opt->id, NULL))
-					{
-						return OPT_ERROR;
-					}
-				}
 			}
-			return opt->id;
-		}
-		id = Opt_CheckBracketValue(opt, str);
-		if (id == OPT_ERROR)
-		{
-			break;
-		}
-		if (id != OPT_CONTINUE)
-		{
-			return id;
 		}
+		return opt->id;
 	}
 	Opt_ShowError(OPT_ERROR, argv[idx], "Unrecognized option");
 	return OPT_ERROR;
@@ -606,6 +661,14 @@ static int Opt_WhichOption(int argc, const char *argv[], int idx)
  */
 static bool Opt_StrCpy(int optid, bool checkexist, char *dst, const char *src, size_t dstlen, bool *option)
 {
+	if (option)
+	{
+		*option = false;
+		if(strcasecmp(src, "none") == 0)
+		{
+			return true;
+		}
+	}
 	if (strlen(src) >= dstlen)
 	{
 		return Opt_ShowError(optid, src, "File name too long!");
@@ -616,30 +679,38 @@ static bool Opt_StrCpy(int optid, bool checkexist, char *dst, const char *src, s
 	}
 	if (option)
 	{
-		if(strcmp(src, "none") == 0)
-		{
-			*option = false;
-			return true;
-		}
-		else
-		{
-			*option = true;
-		}
+		*option = true;
 	}
 	strcpy(dst, src);
 	return true;
 }
 
 
+/**
+ * Return SDL_INIT_NOPARACHUTE flag if user requested SDL parachute
+ * to be disabled to get proper Hatari core dumps.  By default returns
+ * zero so that SDL parachute will be used to restore video mode on
+ * unclean Hatari termination.
+ */
+Uint32 Opt_GetNoParachuteFlag(void)
+{
+	if (bNoSDLParachute) {
+		return SDL_INIT_NOPARACHUTE;
+	}
+	return 0;
+}
+
+
 /**
  * parse all Hatari command line options and set Hatari state accordingly.
  * Returns true if everything was OK, false otherwise.
  */
 bool Opt_ParseParameters(int argc, const char *argv[])
 {
-	int ncpu, skips, zoom, planes, cpuclock, threshold, memsize, port, freq;
+	int ncpu, skips, zoom, planes, cpuclock, threshold, memsize, port, freq, temp;
 	const char *errstr;
 	int i, ok = true;
+	int val;
 
 	/* Defaults for loading initial memory snap-shots */
 	bLoadMemorySave = false;
@@ -684,6 +755,7 @@ bool Opt_ParseParameters(int argc, const char *argv[])
 			
 		case OPT_CONFIGFILE:
 			i += 1;
+			/* true -> file needs to exist */
 			ok = Opt_StrCpy(OPT_CONFIGFILE, true, sConfigFileName,
 					argv[i], sizeof(sConfigFileName), NULL);
 			if (ok)
@@ -735,7 +807,7 @@ bool Opt_ParseParameters(int argc, const char *argv[])
 		case OPT_GRAB:
 			bGrabMouse = true;
 			break;
-			
+
 		case OPT_ZOOM:
 			zoom = atoi(argv[++i]);
 			if (zoom < 1)
@@ -744,15 +816,28 @@ bool Opt_ParseParameters(int argc, const char *argv[])
 			}
 			if (zoom > 1)
 			{
-				/* TODO: only doubling supported for now */
-				ConfigureParams.Screen.bZoomLowRes = true;
+				ConfigureParams.Screen.nMaxWidth = 2*(48+320+48);
+				ConfigureParams.Screen.nMaxHeight = 2*NUM_VISIBLE_LINES+24;
 			}
 			else
 			{
-				ConfigureParams.Screen.bZoomLowRes = false;
+				ConfigureParams.Screen.nMaxWidth = 320;
+				ConfigureParams.Screen.nMaxHeight = 200;
 			}
 			break;
 			
+		case OPT_MAXWIDTH:
+			ConfigureParams.Screen.nMaxWidth = atoi(argv[++i]);
+			break;
+
+		case OPT_MAXHEIGHT:
+			ConfigureParams.Screen.nMaxHeight = atoi(argv[++i]);
+			break;
+			
+		case OPT_ASPECT:
+			ok = Opt_Bool(argv[++i], OPT_ASPECT, &ConfigureParams.Screen.bAspectCorrect);
+			break;
+			
 		case OPT_FRAMESKIPS:
 			skips = atoi(argv[++i]);
 			if (skips < 0)
@@ -807,6 +892,48 @@ bool Opt_ParseParameters(int argc, const char *argv[])
 			ConfigureParams.Screen.nForceBpp = planes;
 			break;
 
+			/* avi recording options */
+		case OPT_AVIRECORD:
+			AviRecordOnStartup = true;
+			break;
+
+		case OPT_AVIRECORD_VCODEC:
+			i += 1;
+			if (strcasecmp(argv[i], "bmp") == 0)
+			{
+				AviRecordDefaultVcodec = AVI_RECORD_VIDEO_CODEC_BMP;
+			}
+			else if (strcasecmp(argv[i], "png") == 0)
+			{
+				AviRecordDefaultVcodec = AVI_RECORD_VIDEO_CODEC_PNG;
+			}
+			else
+			{
+				return Opt_ShowError(OPT_AVIRECORD_VCODEC, argv[i], "Unknown video codec");
+			}
+			break;
+
+		case OPT_AVIRECORD_FPS:
+			val = atoi(argv[++i]);
+			if (val < 0 || val > 100)
+			{
+				return Opt_ShowError(OPT_AVIRECORD_FPS, argv[i],
+							"Invalid frame rate for avi recording");
+			}
+			AviRecordDefaultFps = val;
+			break;
+
+		case OPT_AVIRECORD_CROP:
+			ok = Opt_Bool(argv[++i], OPT_AVIRECORD_CROP, &AviRecordDefaultCrop);
+			break;
+
+		case OPT_AVIRECORD_FILE:
+			i += 1;
+			/* false -> file is created if it doesn't exist */
+			ok = Opt_StrCpy(OPT_AVIRECORD_FILE, false, AviRecordFile,
+					argv[i], sizeof(AviRecordFile), NULL);
+			break;
+
 			/* VDI options */
 		case OPT_VDI:
 			ok = Opt_Bool(argv[++i], OPT_VDI, &ConfigureParams.Screen.bUseExtVdiResolutions);
@@ -847,7 +974,7 @@ bool Opt_ParseParameters(int argc, const char *argv[])
 			ConfigureParams.Screen.bUseExtVdiResolutions = true;
 			bLoadAutoSave = false;
 			break;
-		
+
 			/* devices options */
 		case OPT_JOYSTICK:
 			i++;
@@ -887,6 +1014,7 @@ bool Opt_ParseParameters(int argc, const char *argv[])
 			
 		case OPT_PRINTER:
 			i += 1;
+			/* "none" can be used to disable printer */
 			ok = Opt_StrCpy(OPT_PRINTER, false, ConfigureParams.Printer.szPrintToFileName,
 					argv[i], sizeof(ConfigureParams.Printer.szPrintToFileName),
 					&ConfigureParams.Printer.bEnablePrinting);
@@ -894,15 +1022,14 @@ bool Opt_ParseParameters(int argc, const char *argv[])
 			
 		case OPT_MIDI_IN:
 			i += 1;
-			/* false: "" can be used to disable midi input */
-			ok = Opt_StrCpy(OPT_MIDI_IN, false, ConfigureParams.Midi.sMidiInFileName,
+			ok = Opt_StrCpy(OPT_MIDI_IN, true, ConfigureParams.Midi.sMidiInFileName,
 					argv[i], sizeof(ConfigureParams.Midi.sMidiInFileName),
 					&ConfigureParams.Midi.bEnableMidi);
 			break;
 			
 		case OPT_MIDI_OUT:
 			i += 1;
-			ok = Opt_StrCpy(OPT_MIDI_OUT, true, ConfigureParams.Midi.sMidiOutFileName,
+			ok = Opt_StrCpy(OPT_MIDI_OUT, false, ConfigureParams.Midi.sMidiOutFileName,
 					argv[i], sizeof(ConfigureParams.Midi.sMidiOutFileName),
 					&ConfigureParams.Midi.bEnableMidi);
 			break;
@@ -916,7 +1043,7 @@ bool Opt_ParseParameters(int argc, const char *argv[])
       
 		case OPT_RS232_OUT:
 			i += 1;
-			ok = Opt_StrCpy(OPT_RS232_OUT, true, ConfigureParams.RS232.szOutFileName,
+			ok = Opt_StrCpy(OPT_RS232_OUT, false, ConfigureParams.RS232.szOutFileName,
 					argv[i], sizeof(ConfigureParams.RS232.szOutFileName),
 					&ConfigureParams.RS232.bEnableRS232);
 			break;
@@ -941,6 +1068,34 @@ bool Opt_ParseParameters(int argc, const char *argv[])
 				return Opt_ShowError(OPT_ERROR, argv[i], "Not a disk image");
 			break;
 
+		case OPT_SLOWFLOPPY:
+			ok = Opt_Bool(argv[++i], OPT_SLOWFLOPPY, &ConfigureParams.DiskImage.bSlowFloppy);
+			break;
+
+		case OPT_WRITEPROT_FLOPPY:
+			i += 1;
+			if (strcasecmp(argv[i], "off") == 0)
+				ConfigureParams.DiskImage.nWriteProtection = WRITEPROT_OFF;
+			else if (strcasecmp(argv[i], "on") == 0)
+				ConfigureParams.DiskImage.nWriteProtection = WRITEPROT_ON;
+			else if (strcasecmp(argv[i], "auto") == 0)
+				ConfigureParams.DiskImage.nWriteProtection = WRITEPROT_AUTO;
+			else
+				return Opt_ShowError(OPT_WRITEPROT_FLOPPY, argv[i], "Unknown option value");
+			break;
+
+		case OPT_WRITEPROT_HD:
+			i += 1;
+			if (strcasecmp(argv[i], "off") == 0)
+				ConfigureParams.HardDisk.nWriteProtection = WRITEPROT_OFF;
+			else if (strcasecmp(argv[i], "on") == 0)
+				ConfigureParams.HardDisk.nWriteProtection = WRITEPROT_ON;
+			else if (strcasecmp(argv[i], "auto") == 0)
+				ConfigureParams.HardDisk.nWriteProtection = WRITEPROT_AUTO;
+			else
+				return Opt_ShowError(OPT_WRITEPROT_HD, argv[i], "Unknown option value");
+			break;
+
 		case OPT_HARDDRIVE:
 			i += 1;
 			ok = Opt_StrCpy(OPT_HARDDRIVE, false, ConfigureParams.HardDisk.szHardDiskDirectories[0],
@@ -964,19 +1119,26 @@ bool Opt_ParseParameters(int argc, const char *argv[])
 			}
 			break;
 			
-		case OPT_IDEHDIMAGE:
+		case OPT_IDEMASTERHDIMAGE:
 			i += 1;
-			ok = Opt_StrCpy(OPT_IDEHDIMAGE, true, ConfigureParams.HardDisk.szIdeHardDiskImage,
-					argv[i], sizeof(ConfigureParams.HardDisk.szIdeHardDiskImage),
-					&ConfigureParams.HardDisk.bUseIdeHardDiskImage);
+			ok = Opt_StrCpy(OPT_IDEMASTERHDIMAGE, true, ConfigureParams.HardDisk.szIdeMasterHardDiskImage,
+					argv[i], sizeof(ConfigureParams.HardDisk.szIdeMasterHardDiskImage),
+					&ConfigureParams.HardDisk.bUseIdeMasterHardDiskImage);
 			if (ok)
 			{
 				bLoadAutoSave = false;
 			}
 			break;
 
-		case OPT_SLOWFLOPPY:
-			ok = Opt_Bool(argv[++i], OPT_SLOWFLOPPY, &ConfigureParams.DiskImage.bSlowFloppy);
+		case OPT_IDESLAVEHDIMAGE:
+			i += 1;
+			ok = Opt_StrCpy(OPT_IDESLAVEHDIMAGE, true, ConfigureParams.HardDisk.szIdeSlaveHardDiskImage,
+					argv[i], sizeof(ConfigureParams.HardDisk.szIdeSlaveHardDiskImage),
+					&ConfigureParams.HardDisk.bUseIdeSlaveHardDiskImage);
+			if (ok)
+			{
+				bLoadAutoSave = false;
+			}
 			break;
 			
 			/* Memory options */
@@ -1143,6 +1305,18 @@ bool Opt_ParseParameters(int argc, const char *argv[])
 			}
 			break;
 
+		case OPT_SOUNDBUFFERSIZE:
+			i += 1;
+			temp = atoi(argv[i]);
+			if ( temp == 0 )			/* use default setting for SDL */
+				;
+			else if (temp < 10 || temp > 100)
+				{
+					return Opt_ShowError(OPT_SOUNDBUFFERSIZE, argv[i], "Unsupported sound buffer size");
+				}
+			ConfigureParams.Sound.SdlAudioBufferSize = temp;
+			break;
+
 		case OPT_KEYMAPFILE:
 			i += 1;
 			ok = Opt_StrCpy(OPT_KEYMAPFILE, true, ConfigureParams.Keyboard.szMappingFileName,
@@ -1168,6 +1342,10 @@ bool Opt_ParseParameters(int argc, const char *argv[])
 			}
 			break;
 
+		case OPT_PARACHUTE:
+			bNoSDLParachute = true;
+			break;
+
 		case OPT_BIOSINTERCEPT:
 			bBiosIntercept = true;
 			break;
@@ -1177,6 +1355,10 @@ bool Opt_ParseParameters(int argc, const char *argv[])
 			errstr = Log_SetTraceOptions(argv[i]);
 			if (errstr)
 			{
+				if (!errstr[0]) {
+					/* silent parsing termination */
+					return false;
+				}
 				return Opt_ShowError(OPT_TRACE, argv[i], errstr);
 			}
 			break;
@@ -1204,6 +1386,17 @@ bool Opt_ParseParameters(int argc, const char *argv[])
 					NULL);
 			break;
 
+		case OPT_PARSE:
+			i += 1;
+			ok = DebugUI_SetParseFile(argv[i]);
+			break;
+
+		case OPT_SAVECONFIG:
+			/* Hatari-UI needs Hatari config to start */
+			Configuration_Save();
+			exit(0);
+			break;
+
 		case OPT_LOGLEVEL:
 			i += 1;
 			ConfigureParams.Log.nTextLogLevel = Log_ParseOptions(argv[i]);
@@ -1242,3 +1435,27 @@ bool Opt_ParseParameters(int argc, const char *argv[])
 
 	return true;
 }
+
+/**
+ * Readline match callback for option name completion.
+ * STATE = 0 -> different text from previous one.
+ * Return next match or NULL if no matches.
+ */
+char *Opt_MatchOption(const char *text, int state)
+{
+	static int i, len;
+	const char *name;
+	
+	if (!state) {
+		/* first match */
+		len = strlen(text);
+		i = 0;
+	}
+	/* next match */
+	while (i < ARRAYSIZE(HatariOptions)) {
+		name = HatariOptions[i++].str;
+		if (name && strncasecmp(name, text, len) == 0)
+			return (strdup(name));
+	}
+	return NULL;
+}
