@@ -12,11 +12,12 @@ const char Reset_fileid[] = "Hatari reset.c : " __DATE__ " " __TIME__;
 #include "configuration.h"
 #include "cart.h"
 #include "dmaSnd.h"
+#include "crossbar.h"
 #include "fdc.h"
 #include "floppy.h"
 #include "gemdos.h"
 #include "ikbd.h"
-#include "int.h"
+#include "cycInt.h"
 #include "m68000.h"
 #include "mfp.h"
 #include "midi.h"
@@ -50,7 +51,7 @@ static int Reset_ST(bool bCold)
 
 		Cart_ResetImage();          /* Load cartridge program into ROM memory. */
 	}
-	Int_Reset();                  /* Reset interrupts */
+	CycInt_Reset();               /* Reset interrupts */
 	MFP_Reset();                  /* Setup MFP chip */
 	Video_Reset();                /* Reset video */
 
@@ -60,7 +61,11 @@ static int Reset_ST(bool bCold)
 		FDC_Reset();                /* Reset FDC */
 	}
 
-	DmaSnd_Reset(bCold);          /* Reset DMA sound */
+	if (ConfigureParams.System.nMachineType == MACHINE_FALCON)
+		Crossbar_Reset(bCold);        /* Reset Crossbar sound */
+	else
+		DmaSnd_Reset(bCold);          /* Reset DMA sound */
+
 	PSG_Reset();                  /* Reset PSG */
 	Sound_Reset();                /* Reset Sound */
 	IKBD_Reset(bCold);            /* Keyboard */
