@@ -38,6 +38,7 @@ const char Screen_fileid[] = "Hatari screen.c : " __DATE__ " " __TIME__;
 #include "screen.h"
 #include "control.h"
 #include "convert/routines.h"
+#include "resolution.h"
 #include "sound.h"
 #include "spec512.h"
 #include "statusbar.h"
@@ -91,6 +92,9 @@ static bool bScrDoubleY;                /* true if double on Y */
 static int ScrUpdateFlag;               /* Bit mask of how to update screen */
 
 
+static bool Screen_DrawFrame(bool bForceFlip);
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * Create ST 0x777 / STe 0xfff color format to 16 or 32 bits per pixel
@@ -229,12 +233,12 @@ static void Screen_Handle8BitPalettes(void)
 /**
  * Set screen draw functions.
  */
-static void Screen_SetDrawFunctions(int nBitCount)
+static void Screen_SetDrawFunctions(int nBitCount, bool bDoubleLowRes)
 {
 	if (nBitCount == 8)
 	{
 		/* Low color */
-		if (ConfigureParams.Screen.bZoomLowRes)
+		if (bDoubleLowRes)
 			ScreenDrawFunctionsNormal[ST_LOW_RES] = ConvertLowRes_640x8Bit;
 		else
 			ScreenDrawFunctionsNormal[ST_LOW_RES] = ConvertLowRes_320x8Bit;
@@ -244,7 +248,7 @@ static void Screen_SetDrawFunctions(int nBitCount)
 	else if (nBitCount <= 16)
 	{
 		/* High color */
-		if (ConfigureParams.Screen.bZoomLowRes)
+		if (bDoubleLowRes)
 			ScreenDrawFunctionsNormal[ST_LOW_RES] = ConvertLowRes_640x16Bit;
 		else
 			ScreenDrawFunctionsNormal[ST_LOW_RES] = ConvertLowRes_320x16Bit;
@@ -254,7 +258,7 @@ static void Screen_SetDrawFunctions(int nBitCount)
 	else /* Assume 32 bit drawing functions */
 	{
 		/* True color */
-		if (ConfigureParams.Screen.bZoomLowRes)
+		if (bDoubleLowRes)
 			ScreenDrawFunctionsNormal[ST_LOW_RES] = ConvertLowRes_640x32Bit;
 		else
 			ScreenDrawFunctionsNormal[ST_LOW_RES] = ConvertLowRes_320x32Bit;
@@ -266,40 +270,50 @@ static void Screen_SetDrawFunctions(int nBitCount)
 
 /*-----------------------------------------------------------------------*/
 /**
- * Set amount of border pixels for windowed and full-screen mode and
+ * Set amount of border pixels
+ */
+static void Screen_SetBorderPixels(int leftX, int leftY)
+{
+	/* All screen widths need to be aligned to 16-bits.
+	 * 
+	 * TODO: Change VDI_Limit() to generic function and check
+	 * the limits when config values are set or changed
+	 */
+	nBorderPixelsLeft = VDI_Limit(leftX/2, 16, 0, 48);
+	nBorderPixelsRight = nBorderPixelsLeft;
+
+	/* assertain assumption of code below */
+	assert(OVERSCAN_TOP < MAX_OVERSCAN_BOTTOM);
+	
+	if (leftY > 2*OVERSCAN_TOP)
+	{
+		nBorderPixelsTop = OVERSCAN_TOP;
+		if (leftY >= OVERSCAN_TOP + MAX_OVERSCAN_BOTTOM)
+			nBorderPixelsBottom = MAX_OVERSCAN_BOTTOM;
+		else
+			nBorderPixelsBottom = leftY - OVERSCAN_TOP;
+	}
+	else
+	{
+		if (leftY > 0)
+			nBorderPixelsTop = nBorderPixelsBottom = leftY/2;
+		else
+			nBorderPixelsTop = nBorderPixelsBottom = 0;
+	}
+}
+
+/*-----------------------------------------------------------------------*/
+/**
  * store Y offset for each horizontal line in our source ST screen for
- * reference in tje convert functions.
+ * reference in the convert functions.
  */
 static void Screen_SetSTScreenOffsets(void)
 {
 	int i;
 
-	/* Determine border pixels */
-	if (bInFullScreen)
-	{
-		nBorderPixelsTop = ConfigureParams.Screen.nFullScreenBorderPixelsTop;
-		nBorderPixelsLeft = ConfigureParams.Screen.nFullScreenBorderPixelsLeft;
-		nBorderPixelsRight = ConfigureParams.Screen.nFullScreenBorderPixelsRight;
-		nBorderPixelsBottom = ConfigureParams.Screen.nFullScreenBorderPixelsBottom;
-	}
-	else
-	{
-		nBorderPixelsTop = ConfigureParams.Screen.nWindowBorderPixelsTop;
-		nBorderPixelsLeft = ConfigureParams.Screen.nWindowBorderPixelsLeft;
-		nBorderPixelsRight = ConfigureParams.Screen.nWindowBorderPixelsRight;
-		nBorderPixelsBottom = ConfigureParams.Screen.nWindowBorderPixelsBottom;
-	}
-	/* All screen widths need to be aligned to 16-bits.
-	 * 
-	 * TODO: Change VDI_Limit() to generic function and check
-	 * the limits when config values are set or changed (after 1.1).
+	/* Store offset to each horizontal line, uses
+	 * nBorderPixels* variables.
 	 */
-	nBorderPixelsTop = VDI_Limit(nBorderPixelsTop, 1, 0, OVERSCAN_TOP);
-	nBorderPixelsLeft = VDI_Limit(nBorderPixelsLeft, 16, 0, 48);
-	nBorderPixelsRight = VDI_Limit(nBorderPixelsRight, 16, 0, 48);
-	nBorderPixelsBottom = VDI_Limit(nBorderPixelsBottom, 1, 0, MAX_OVERSCAN_BOTTOM);
-
-	/* Store offset to each horizontal line */
 	for (i = 0; i < NUM_VISIBLE_LINES; i++)
 	{
 		STScreenLineOffset[i] = i * SCREENBYTES_LINE;
@@ -313,39 +327,9 @@ static void Screen_SetSTScreenOffsets(void)
  */
 static void Screen_SetResolution(void)
 {
-	int Width, Height, BitCount;
-	unsigned int sdlVideoFlags;
-
-	Screen_SetSTScreenOffsets();  
-
-	/* Determine which resolution to use */
-	if (bUseVDIRes)
-	{
-		Width = VDIWidth;
-		Height = VDIHeight;
-	}
-	else
-	{
-		if (STRes == ST_LOW_RES && !ConfigureParams.Screen.bZoomLowRes)
-		{
-			Width = 320;
-			Height = 200;
-		}
-		else    /* else use 640x400 */
-		{
-			Width = 640;
-			Height = 400;
-		}
-
-		/* Adjust width/height for overscan borders, if mono or VDI we have no overscan */
-		if (ConfigureParams.Screen.bAllowOverscan && !bUseHighRes)
-		{
-			int nZoom = ((Width == 640) ? 2 : 1);
-			/* Add in overscan borders (if 640x200 bitmap is double on Y) */
-			Width += (nBorderPixelsLeft+nBorderPixelsRight) * nZoom;
-			Height += (nBorderPixelsTop+nBorderPixelsBottom) * nZoom;
-		}
-	}
+	int Width, Height, nZoom, SBarHeight, BitCount, maxW, maxH;
+	Uint32 sdlVideoFlags;
+	bool bDoubleLowRes = false;
 
 	/* Bits per pixel */
 	if (STRes == ST_HIGH_RES || bUseVDIRes)
@@ -357,41 +341,99 @@ static void Screen_SetResolution(void)
 		BitCount = ConfigureParams.Screen.nForceBpp;
 	}
 
-	/* Set zoom factors, used for scaling mouse motions */
-	if (STRes == ST_LOW_RES && ConfigureParams.Screen.bZoomLowRes && !bUseVDIRes)
+	nBorderPixelsTop = nBorderPixelsBottom = 0;
+	nBorderPixelsLeft = nBorderPixelsRight = 0;
+
+	nScreenZoomX = 1;
+	nScreenZoomY = 1;
+
+	/* Determine which resolution to use */
+	if (bUseVDIRes)
 	{
-		nScreenZoomX = 2;
-		nScreenZoomY = 2;
-	}
-	else if (STRes == ST_MEDIUM_RES && !bUseVDIRes)
-	{
-		nScreenZoomX = 1;
-		nScreenZoomY = 2;
+		Width = VDIWidth;
+		Height = VDIHeight;
 	}
 	else
 	{
-		nScreenZoomX = 1;
-		nScreenZoomY = 1;
+		if (STRes == ST_LOW_RES)
+ 		{
+ 			Width = 320;
+ 			Height = 200;
+			nZoom = 1;
+ 		}
+		else    /* else use 640x400, also for med-rez */
+ 		{
+ 			Width = 640;
+ 			Height = 400;
+			nZoom = 2;
+ 		}
+
+		/* Statusbar height for doubled screen size */
+		SBarHeight = Statusbar_GetHeightForSize(640, 400);
+
+		Resolution_GetLimits(&maxW, &maxH, &BitCount);
+		
+		/* Zoom if necessary, factors used for scaling mouse motions */
+		if (STRes == ST_LOW_RES &&
+		    2*Width <= maxW && 2*Height+SBarHeight <= maxH)
+		{
+			nZoom = 2;
+			Width *= 2;
+			Height *= 2;
+			nScreenZoomX = 2;
+			nScreenZoomY = 2;
+			bDoubleLowRes = true;
+		}
+		else if (STRes == ST_MEDIUM_RES)
+		{
+			/* med-rez conversion functions want always
+			 * to double vertically, they don't support
+			 * skipping that (only leaving doubled lines
+			 * black for the TV mode).
+			 */
+			nScreenZoomX = 1;
+			nScreenZoomY = 2;
+ 		}
+
+		/* Adjust width/height for overscan borders, if mono or VDI we have no overscan */
+		if (ConfigureParams.Screen.bAllowOverscan && !bUseHighRes)
+		{
+			int leftX = maxW - Width;
+			int leftY = maxH - (Height + Statusbar_GetHeightForSize(Width, Height));
+
+			Screen_SetBorderPixels(leftX/nZoom, leftY/nZoom);
+			Width += (nBorderPixelsRight + nBorderPixelsLeft)*nZoom;
+			Height += (nBorderPixelsTop + nBorderPixelsBottom)*nZoom;
+		}
 	}
+	
+	Screen_SetSTScreenOffsets();  
+	Height += Statusbar_SetHeight(Width, Height);
 
 	/* SDL Video attributes: */
 	if (bInFullScreen)
 	{
 		sdlVideoFlags  = SDL_HWSURFACE|SDL_FULLSCREEN|SDL_HWPALETTE/*|SDL_DOUBLEBUF*/;
-		/* SDL_DOUBLEBUF is a good idea, but the GUI doesn't work with double buffered
-		 * screens yet, so double buffering is currently disabled. */
+		/* SDL_DOUBLEBUF helps avoiding tearing and can be faster on suitable HW,
+		 * but it doesn't work with partial screen updates done by the ST screen
+		 * update code or the Hatari GUI, so double buffering is disabled.
+		 */
 	}
 	else
 	{
 		sdlVideoFlags  = SDL_SWSURFACE|SDL_HWPALETTE;
 	}
-	Height += Statusbar_SetHeight(Width, Height);
 
 	/* Check if we really have to change the video mode: */
 	if (!sdlscrn || sdlscrn->w != Width || sdlscrn->h != Height
-	    || sdlscrn->format->BitsPerPixel != BitCount
+	    || (BitCount && sdlscrn->format->BitsPerPixel != BitCount)
 	    || (sdlscrn->flags&SDL_FULLSCREEN) != (sdlVideoFlags&SDL_FULLSCREEN))
 	{
+#ifdef _MUDFLAP
+		if (sdlscrn) {
+			__mf_unregister(sdlscrn->pixels, sdlscrn->pitch*sdlscrn->h, __MF_TYPE_GUESS);
+		}
+#endif
 		if (bInFullScreen)
 		{
 			/* unhide the Hatari WM window for fullscreen */
@@ -421,6 +463,9 @@ static void Screen_SetResolution(void)
 			SDL_Quit();
 			exit(-2);
 		}
+#ifdef _MUDFLAP
+		__mf_register(sdlscrn->pixels, sdlscrn->pitch*sdlscrn->h, __MF_TYPE_GUESS, "SDL pixels");
+#endif
 
 		if (!bInFullScreen)
 		{
@@ -444,7 +489,7 @@ static void Screen_SetResolution(void)
 	}
 
 	/* Set drawing functions */
-	Screen_SetDrawFunctions(sdlscrn->format->BitsPerPixel);
+	Screen_SetDrawFunctions(sdlscrn->format->BitsPerPixel, bDoubleLowRes);
 
 	Screen_SetFullUpdate();           /* Cause full update of screen */
 }
@@ -572,19 +617,59 @@ static void Screen_ClearScreen(void)
 	SDL_FillRect(sdlscrn, &STScreenRect, SDL_MapRGB(sdlscrn->format, 0, 0, 0));
 }
 
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Return true if (falcon/tt) hostscreen functions need to be used
+ * instead of the (st/ste) functions here.
+ */
+static bool Screen_UseHostScreen(void)
+{
+	return ((ConfigureParams.System.nMachineType == MACHINE_FALCON
+		 || ConfigureParams.System.nMachineType == MACHINE_TT)
+		&& !bUseVDIRes);
+}
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Force screen redraw.  Does the right thing regardless of whether
+ * we're in ST/STe, Falcon or TT mode.  Needed when switching modes
+ * while emulation is paused.
+ */
+static void Screen_Refresh(void)
+{
+	if (!bUseVDIRes)
+	{
+		if (ConfigureParams.System.nMachineType == MACHINE_FALCON)
+		{
+			VIDEL_renderScreen();
+			return;
+		}
+		else if (ConfigureParams.System.nMachineType == MACHINE_TT)
+		{
+			Video_RenderTTScreen();
+			return;
+		}
+	}
+	Screen_DrawFrame(true);
+}
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * Enter Full screen mode
  */
 void Screen_EnterFullScreen(void)
 {
+	bool bWasRunning;
+
 	if (!bInFullScreen)
 	{
-		Main_PauseEmulation(false);         /* Hold things... */
+		/* Hold things... */
+		bWasRunning = Main_PauseEmulation(false);
 		bInFullScreen = true;
 
-		if ((ConfigureParams.System.nMachineType == MACHINE_FALCON
-		     || ConfigureParams.System.nMachineType == MACHINE_TT) && !bUseVDIRes)
+		if (Screen_UseHostScreen())
 		{
 			HostScreen_toggleFullScreen();
 		}
@@ -595,8 +680,16 @@ void Screen_EnterFullScreen(void)
 		}
 
 		SDL_Delay(20);                  /* To give monitor time to change to new resolution */
-		Main_UnPauseEmulation();        /* And off we go... */
-
+		
+		if (bWasRunning)
+		{
+			/* And off we go... */
+			Main_UnPauseEmulation();
+		}
+		else
+		{
+			Screen_Refresh();
+		}
 		SDL_WM_GrabInput(SDL_GRAB_ON);  /* Grab mouse pointer in fullscreen */
 	}
 }
@@ -608,13 +701,15 @@ void Screen_EnterFullScreen(void)
  */
 void Screen_ReturnFromFullScreen(void)
 {
+	bool bWasRunning;
+
 	if (bInFullScreen)
 	{
-		Main_PauseEmulation(false);        /* Hold things... */
+		/* Hold things... */
+		bWasRunning = Main_PauseEmulation(false);
 		bInFullScreen = false;
 
-		if ((ConfigureParams.System.nMachineType == MACHINE_FALCON
-		     || ConfigureParams.System.nMachineType == MACHINE_TT) && !bUseVDIRes)
+		if (Screen_UseHostScreen())
 		{
 			HostScreen_toggleFullScreen();
 		}
@@ -623,11 +718,22 @@ void Screen_ReturnFromFullScreen(void)
 			Screen_SetResolution();
 		}
 		SDL_Delay(20);                /* To give monitor time to switch resolution */
-		Main_UnPauseEmulation();      /* And off we go... */
+
+		if (bWasRunning)
+		{
+			/* And off we go... */
+			Main_UnPauseEmulation();
+		}
+		else
+		{
+			Screen_Refresh();
+		}
 
 		if (!bGrabMouse)
+		{
 			/* Un-grab mouse pointer in windowed mode */
 			SDL_WM_GrabInput(SDL_GRAB_OFF);
+		}
 	}
 }
 
@@ -932,24 +1038,6 @@ static void Screen_UnLock(void)
 }
 
 
-/*-----------------------------------------------------------------------*/
-/**
- * Swap ST Buffers, used for full-screen where have double-buffering
- */
-static void Screen_SwapSTBuffers(void)
-{
-#if NUM_FRAMEBUFFERS > 1
-	if (sdlscrn->flags & SDL_DOUBLEBUF)
-	{
-		if (pFrameBuffer==&FrameBuffers[0])
-			pFrameBuffer = &FrameBuffers[1];
-		else
-			pFrameBuffer = &FrameBuffers[0];
-	}
-#endif
-}
-
-
 /*-----------------------------------------------------------------------*/
 /**
  * Blit our converted ST screen to window/full-screen
@@ -958,14 +1046,20 @@ static void Screen_Blit(void)
 {
 	unsigned char *pTmpScreen;
 
-	/* Blit to full screen or window? */
-	if (bInFullScreen)
+#if 0	/* double buffering cannot be used with partial screen updates */
+# if NUM_FRAMEBUFFERS > 1
+	if (bInFullScreen && (sdlscrn->flags & SDL_DOUBLEBUF))
 	{
-		Screen_SwapSTBuffers();
 		/* Swap screen */
+		if (pFrameBuffer==&FrameBuffers[0])
+			pFrameBuffer = &FrameBuffers[1];
+		else
+			pFrameBuffer = &FrameBuffers[0];
 		SDL_Flip(sdlscrn);
 	}
 	else
+# endif
+#endif
 	{
 		SDL_UpdateRects(sdlscrn, 1, &STScreenRect);
 	}
